   These programs are here mostly as examples to be looked at.  But they all
do execute; they are the test bed for my own changes.


  To run all of these tests-
	Change 'setup.s' as appropriate.  The one contained here is based
    on the assumption that the parent directory contains all of the
    .o files, and ../.Data all of the new functions.
	Change "Splus" to "S" in testall.sh, if appropriate.
	Then
		mkdir .Data
		sh testall.sh

   A batch job will be spawned that will create testall.out.  You can compare
this with testall.save.

   The file data.bladder4 is just bladder, restructured to make it easier
to read.
